include "my_database.iol"
include "message_digest.iol"

execution { concurrent }

type CreateUserRequest:void {
	.email:string
	.password:string
}

type DumpTableRequest:void {
	.table:string
}

type Credentials:void {
	.email:string
	.password:string
}

interface MyInterface {
RequestResponse:
	createUser(CreateUserRequest)(void)
		throws UserAlreadyExists(void),
	dumpTable(DumpTableRequest)(undefined),
	authenticate(Credentials)(void)
		throws InvalidCredentials(void)
}

inputPort MyInput {
Location: "socket://localhost:8000/"
Protocol: http
Interfaces: MyInterface
}

main
{
	[ createUser( req )() {
		c = "select email from users
		where email=:email";
		c.email = req.email;
		query@Database( c )( rec );
		if ( #rec.row > 0 ) {
			throw( UserAlreadyExists )
		};
		q = "insert into
		users values
		(
			default,
			:email,
			:password
		)";
		q.email = req.email;
		q.password = req.password;
		md5@MessageDigest( q.password )( q.password );
		update@Database( q )()
	} ] { nullProcess }

	[ dumpTable( req )( res ) {
		q = "select * from " + req.table;
		query@Database( q )( res )
	} ] { nullProcess }

	[ authenticate( cred )() {
		q = "select email from users
		where email=:email and
		password=:password";
		q.email = cred.email;
		md5@MessageDigest( cred.password )( q.password );
		query@Database( q )( rec );
		if ( #rec.row > 0 ) {
			nullProcess
		} else {
			throw( InvalidCredentials )
		}
	} ] { nullProcess }
}